@@ -1,5 +1,4 @@
 import asyncio
-import base64
 import os
 from datetime import datetime, timedelta
 from io import BytesIO
@@ -8,7 +7,6 @@ from pathlib import Path
 from PIL import Image
 from nonebot import get_driver, logger
 from nonebot.adapters.onebot.v11 import Message, MessageSegment
-from playwright._impl._api_types import TimeoutError
 
 from .config import Config
 from .._utils import get_browser
@@ -17,15 +15,17 @@ from .._utils import get_browser
 ark_daily_config = Config.parse_obj(get_driver().config.dict())
 DAILY_LEVELS_PATH = Path(ark_daily_config.daily_levels_path)
 
+
 async def get_daily_sources(is_force: bool = False):
     """获取每日资源关卡"""
-    os.mkdir(DAILY_LEVELS_PATH) if not os.path.exists(DAILY_LEVELS_PATH) else None
+    None if os.path.exists(DAILY_LEVELS_PATH) else os.mkdir(DAILY_LEVELS_PATH)
     today = str((datetime.now() - timedelta(hours=4)).date())
     file_name = DAILY_LEVELS_PATH / f"{today}.png"
     if file_name.exists() and not is_force:
-        return Message(MessageSegment.image(file_name) + f"{today} - 数据来源于 https://prts.wiki/")
+        return Message(f"{MessageSegment.image(file_name)}{today} - 数据来源于 https://prts.wiki/")
 
     page = None
+    screenshot = None
     for retry in range(3):
         try:
             browser = await get_browser()
@@ -33,7 +33,7 @@ async def get_daily_sources(is_force: bool = False):
                 return None
             url = "https://prts.wiki/"
             page = await browser.new_page()
-            await page.goto(url, timeout=10000)
+            await page.goto(url, timeout=50000)
             await page.set_viewport_size({"width": 1920, "height": 1080})
             result = await page.query_selector('div[class="mp-container-left"]')
             await page.wait_for_selector("img", state="visible")
@@ -45,11 +45,8 @@ async def get_daily_sources(is_force: bool = False):
             box['height'] += 70
             await asyncio.sleep(1)
             screenshot = await page.screenshot(clip=box)
-        except TimeoutError:
-            logger.warning(f"第{retry + 1}次获取方舟每日资源截图失败……")
-            continue
-        except AttributeError:
-            logger.warning(f"第{retry + 1}次获取方舟每日资源截图失败……")
+        except (TimeoutError, AttributeError) as e:
+            logger.warning(f"第{retry + 1}次获取方舟每日资源截图失败…… {e}")
             continue
         except Exception as e:
             if page:
@@ -58,11 +55,14 @@ async def get_daily_sources(is_force: bool = False):
             return None
         else:
             await page.close()
-            b_scr = BytesIO(screenshot)
-            img = Image.open(b_scr)
+            break
+
+    if screenshot:
+        b_scr = BytesIO(screenshot)
+        with Image.open(b_scr) as img:
             img.save(file_name)
-            return Message(
-                MessageSegment.image(base64.b64encode(screenshot)) + f"{today} - 数据来源于 https://prts.wiki/")
+        return Message(f"{MessageSegment.image(file_name)}{today} - 数据来源于 https://prts.wiki/")
+
     if page:
         await page.close()
-    return None
+    return None
\ No newline at end of file
